# Makefile for g++ library test files for OS/2 2.0 and emx+gcc

GENCLASS = genclass
PROTODIR = ../src/gen

AR = ar
AR_FLAGS = rc
RANLIB = ar s

CC = gcc -g
CFLAGS = -I. -I../iostream/ -I../src
LIBS = -lgpp

XTRAFLAGS =

.SUFFIXES: .o .cc

.cc.o:
	$(CC) $(CFLAGS) $(XTRAFLAGS) -c $<

TOUTS = test_h.exe $(TEST0) tFile.exe tObstack.exe tString.exe \
  tInteger.exe tRational.exe tComplex.exe tBitSet.exe tBitString.exe \
  tRandom.exe tFix.exe tFix16.exe tFix24.exe tGetOpt.exe \
  tList.exe tPlex.exe tLList.exe tVec.exe tStack.exe tQueue.exe \
  tDeque.exe tPQ.exe tSet.exe tBag.exe tMap.exe tCurses.exe 

LHDRS = idefs.h

LOBJS = \
 iList.o iSLList.o iDLList.o iVec.o iAVec.o \
 iPlex.o  iFPlex.o  iXPlex.o iRPlex.o iMPlex.o \
 iSet.o iBag.o iMap.o iPQ.o \
 iXPSet.o  iOXPSet.o  iSLSet.o  iOSLSet.o  iBSTSet.o iCHNode.o \
 iAVLSet.o  iSplayNode.o iSplaySet.o  iVHSet.o  iVOHSet.o  iCHSet.o \
 iXPBag.o  iOXPBag.o  iSLBag.o  iOSLBag.o  iSplayBag.o iVHBag.o  iCHBag.o \
 iVHMap.o  iCHMap.o  iSplayMap.o  iAVLMap.o iRAVLMap.o \
 iSplayPQ.o  iPHPQ.o  iXPPQ.o \
 iVStack.o iVQueue.o iStack.o iQueue.o iDeque.o \
 iXPStack.o iSLStack.o iXPQueue.o  iSLQueue.o iXPDeque.o iDLDeque.o

LIBTEST=libtest.a
TESTS = $(LIBTEST) $(TOUTS)

EXPECTED_OUTPUT = out.iostreams

.PHONY: all
all: $(TESTS)

.PHONY: check
check: tests

$(TOUTS): $(LIBGXX)

test_h.exe: test_h.o
	$(CC) test_h.o -o $@ $(LIBS)

$(LIBTEST): $(LHDRS) $(LOBJS)
	-del $(LIBTEST)
	$(AR) r $(LIBTEST) $(LOBJS)
	$(RANLIB) $(LIBTEST)

# test0

test0: test0.h test.hello.o test.bye test.bye2 test.shell
	$(CC) test.hello.o $(LIBS) -o $@

test.hello.o: test0.h test.hello.cc
	$(CC) $(TFLAGS) test.hello.cc -o $@

test.bye: test0.h test.bye.cc
	$(CC) $(TFLAGS) test.bye.cc  -o $@

test.bye2: test0.h test.bye2.cc
	$(CC) $(TFLAGS) test.bye2.cc  -o $@

test.shell: test0.h test.shell.cc
	$(CC) $(TFLAGS) test.shell.cc  -o $@

# other tests

tFile.exe:  tFile.o
	$(CC) tFile.o -o $@ $(LIBS)

tObstack.exe:  tObstack.o
	$(CC) tObstack.o -o $@ $(LIBS)

tString.exe:  tString.o
	$(CC) tString.o -o $@ $(LIBS)

tInteger.exe:  tInteger.o
	$(CC) tInteger.o -o $@ $(LIBS)

twrapper.exe:  twrapper.o
	$(CC) twrapper.o -o $@ $(LIBS)

tgwrapper.exe:   tgwrapper.o
	$(CC) tgwrapper.o -o $@ $(LIBS)

tRational.exe:  tRational.o
	$(CC) tRational.o -o $@ $(LIBS)

tComplex.exe:  tComplex.o
	$(CC) tComplex.o -o $@ $(LIBS)

tBitSet.exe:  tBitSet.o
	$(CC) tBitSet.o -o $@ $(LIBS)

tBitString.exe:  tBitString.o
	$(CC) tBitString.o -o $@ $(LIBS)

tRandom.exe:  tRandom.o
	$(CC) tRandom.o -o $@ $(LIBS)

tFix.exe:  tFix.o
	$(CC) tFix.o -o $@ $(LIBS)

tFix16.exe:  tFix16.o
	$(CC) tFix16.o -o $@ $(LIBS)

tFix24.exe:  tFix24.o
	$(CC) tFix24.o -o $@ $(LIBS)

tCurses.exe:  tCurses.o
	$(CC)  tCurses.o  -o $@ $(LIBS) -lcurs -ltermc

tGetOpt.exe:  tGetOpt.o
	$(CC) tGetOpt.o -o $@ $(LIBS)

tList.exe:  $(LIBTEST)  tList.o
	$(CC) tList.o -o $@ $(LIBTEST) $(LIBS)

tPlex.exe:  $(LIBTEST)  tPlex.o
	$(CC) tPlex.o -o $@ $(LIBTEST) $(LIBS)

tLList.exe:  $(LIBTEST) tLList.o
	$(CC) tLList.o -o $@ $(LIBTEST) $(LIBS)

tVec.exe: $(LIBTEST) tVec.o
	$(CC) tVec.o -o $@ $(LIBTEST) $(LIBS)

tStack.exe: $(LIBTEST) tStack.o
	$(CC) tStack.o -o $@ $(LIBTEST) $(LIBS)

tQueue.exe: $(LIBTEST) tQueue.o
	$(CC) tQueue.o -o $@ $(LIBTEST) $(LIBS)

tDeque.exe: $(LIBTEST) tDeque.o
	$(CC) tDeque.o -o $@ $(LIBTEST) $(LIBS)

tPQ.exe: $(LIBTEST) tPQ.o
	$(CC) tPQ.o -o $@ $(LIBTEST) $(LIBS)

tSet.exe: $(LIBTEST) tSet.o
	$(CC) tSet.o -o $@ $(LIBTEST) $(LIBS)

tBag.exe:  $(LIBTEST) tBag.o
	$(CC) tBag.o -o $@ $(LIBTEST) $(LIBS)

tMap.exe:  $(LIBTEST) tMap.o
	$(CC) tMap.o -o $@ $(LIBTEST) $(LIBS)

idefs.h:
	$(GENCLASS) int val defs i
iSet.h iSet.cc :
	$(GENCLASS) int val Set i
iXPSet.h iXPSet.cc :
	$(GENCLASS) int val XPSet i
iOXPSet.h iOXPSet.cc :
	$(GENCLASS) int val OXPSet i
iSLSet.h iSLSet.cc :
	$(GENCLASS) int val SLSet i
iOSLSet.h iOSLSet.cc :
	$(GENCLASS) int val OSLSet i
iBSTSet.h iBSTSet.cc :
	$(GENCLASS) int val BSTSet i
iAVLSet.h iAVLSet.cc :
	$(GENCLASS) int val AVLSet i
iSplayNode.h iSplayNode.cc :
	$(GENCLASS) int val SplayNode i
iSplaySet.h iSplaySet.cc :
	$(GENCLASS) int val SplaySet i
iVHSet.h iVHSet.cc :
	$(GENCLASS) int val VHSet i
iVOHSet.h iVOHSet.cc :
	$(GENCLASS) int val VOHSet i
iCHSet.h iCHSet.cc :
	$(GENCLASS) int val CHSet i
iCHNode.h iCHNode.cc :
	$(GENCLASS) int val CHNode i
iBag.h iBag.cc :
	$(GENCLASS) int val Bag i
iXPBag.h iXPBag.cc :
	$(GENCLASS) int val XPBag i
iOXPBag.h iOXPBag.cc :
	$(GENCLASS) int val OXPBag i
iSLBag.h iSLBag.cc :
	$(GENCLASS) int val SLBag i
iOSLBag.h iOSLBag.cc :
	$(GENCLASS) int val OSLBag i
iSplayBag.h iSplayBag.cc :
	$(GENCLASS) int val SplayBag i
iVHBag.h iVHBag.cc :
	$(GENCLASS) int val VHBag i
iCHBag.h iCHBag.cc :
	$(GENCLASS) int val CHBag i
iMap.h iMap.cc :
	$(GENCLASS) -2 int val int val Map i
iVHMap.h iVHMap.cc :
	$(GENCLASS) -2 int val int val VHMap i
iCHMap.h iCHMap.cc :
	$(GENCLASS) -2 int val int val CHMap i
iSplayMap.h iSplayMap.cc :
	$(GENCLASS) -2 int val int val SplayMap i
iAVLMap.h iAVLMap.cc :
	$(GENCLASS) -2 int val int val AVLMap i
iRAVLMap.h iRAVLMap.cc :
	$(GENCLASS) -2 int val int val RAVLMap i
iPQ.h iPQ.cc:
	$(GENCLASS) int val PQ i
iPHPQ.h iPHPQ.cc:
	$(GENCLASS) int val PHPQ i
iSplayPQ.h iSplayPQ.cc:
	$(GENCLASS) int val SplayPQ i
iXPPQ.h iXPPQ.cc:
	$(GENCLASS) int val XPPQ i
iStack.h iStack.cc:
	$(GENCLASS) int val Stack i
iQueue.h iQueue.cc:
	$(GENCLASS) int val Queue i
iDeque.h iDeque.cc:
	$(GENCLASS) int val Deque i
iSLStack.h iSLStack.cc:
	$(GENCLASS) int val SLStack i
iSLQueue.h iSLQueue.cc:
	$(GENCLASS) int val SLQueue i
iDLDeque.h iDLDeque.cc:
	$(GENCLASS) int val DLDeque i
iList.h iList.cc:
	$(GENCLASS) int val List i
iPlex.h iPlex.cc:
	$(GENCLASS) int val Plex i
iFPlex.h iFPlex.cc:
	$(GENCLASS) int val FPlex i
iXPlex.h iXPlex.cc:
	$(GENCLASS) int val XPlex i
iMPlex.h iMPlex.cc:
	$(GENCLASS) int val MPlex i
iRPlex.h iRPlex.cc:
	$(GENCLASS) int val RPlex i
iFPStack.h iFPStack.cc:
	$(GENCLASS) int val FPStack i
iXPStack.h iXPStack.cc:
	$(GENCLASS) int val XPStack i
iFPQueue.h iFPQueue.cc:
	$(GENCLASS) int val FPQueue i
iXPQueue.h iXPQueue.cc:
	$(GENCLASS) int val XPQueue i
iXPDeque.h iXPDeque.cc:
	$(GENCLASS) int val XPDeque i
iSLList.h iSLList.cc:
	$(GENCLASS) int val SLList i
iDLList.h iDLList.cc:
	$(GENCLASS) int val DLList i
iVec.h iVec.cc:
	$(GENCLASS) int val Vec i
iAVec.h iAVec.cc:
	$(GENCLASS) int val AVec i
iVStack.h iVStack.cc: iStack.h
	$(GENCLASS) int val VStack i
iVQueue.h iVQueue.cc: iQueue.h
	$(GENCLASS) int val VQueue i

.PHONY: clean
clean:
	-del i*.cc i*.h *.o *.a *.exe plot.out streamfile ffile
