/* nice -- run a program with modified scheduling priority
   Copyright (C) 1990, 1991 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* David MacKenzie <djm@ai.mit.edu> */

#ifdef OS2

#define NICE

#define INCL_NOPM
#define INCL_DOSPROCESS
#include <os2.h>
#include <errno.h>

#define PRIO_PROCESS 0

int class = PRTYC_NOCHANGE;
int adjustment;
char *current_class;
char *name[] = {"", "idletime", "regular", "timecritical", "foregroundserver"};

int getpriority(int x, int y)
{
#ifdef MSC
  USHORT prio, class;
  DosGetPrty(PRTYS_PROCESS, &prio, 0);
#else
  ULONG prio, class;
  PTIB ptib;
  PPIB ppib;

  DosGetInfoBlocks(&ptib, &ppib);
  prio = ptib -> tib_ptib2 -> tib2_ulpri;
#endif
   
  class = prio >> 8;
  current_class = name[class];
  return -(prio & 255);
}

int setpriority(int x, int y, int v)
{
#ifdef MSC
  return DosSetPrty(PRTYS_PROCESSTREE, class, -adjustment, 0) ? -1 : 0;
#else
  return DosSetPriority(PRTYS_PROCESSTREE, class, -adjustment, 0) ? -1 : 0;
#endif
}

#endif

#include <stdio.h>
#include <getopt.h>
#include <sys/types.h>
#ifndef NICE_PRIORITY
#include <sys/time.h>
#include <sys/resource.h>
#endif
#include "system.h"

int isinteger ();
void error ();
void usage ();

/* The name this program was run with. */
char *program_name;

struct option longopts[] =
{
  {"adjustment", 1, NULL, 'n'},
  {NULL, 0, NULL, 0}
};

void
main (argc, argv)
     int argc;
     char **argv;
{
  int current_priority;
#ifndef OS2
  int adjustment = 0;
#endif
  int minusflag = 0;
  int adjustment_given = 0;
  int optc;

  program_name = argv[0];

#ifdef OS2
  while ((optc = getopt_long (argc, argv, "+0123456789-n:irtf", longopts,
#else
  while ((optc = getopt_long (argc, argv, "+0123456789-n:", longopts,
#endif
			      (int *) 0)) != EOF)
    {
      switch (optc)
	{
	case '?':
	  usage ();

#ifdef OS2
        case 'i':
          class = PRTYC_IDLETIME;
          break;
        case 'r':
          class = PRTYC_REGULAR;
          break;
        case 't':
          class = PRTYC_TIMECRITICAL;
          break;
        case 'f':
          class = PRTYC_FOREGROUNDSERVER;
          break;
#endif

	case 'n':
	  if (!isinteger (optarg))
	    error (1, 0, "invalid priority `%s'", optarg);
	  adjustment = atoi (optarg);
	  adjustment_given = 1;
	  break;

	case '-':
	  minusflag = 1;
	  break;

	default:
	  adjustment = adjustment * 10 + optc - '0';
	  adjustment_given = 1;
	}
    }

  if (minusflag)
    adjustment = -adjustment;
  if (!adjustment_given)
    adjustment = 10;

  if (optind == argc)
    {
      if (adjustment_given)
	usage ();
      /* No command given; print the priority. */
      errno = 0;
#ifndef NICE_PRIORITY
      current_priority = getpriority (PRIO_PROCESS, 0);
#else
      current_priority = nice (0);
#endif
      if (current_priority == -1 && errno != 0)
	error (1, errno, "cannot get priority");
#ifdef OS2
      printf ("\ncurrent class: %s\ncurrent priority: %d\n",
              current_class, current_priority);
      usage ();
#else
      printf ("%d\n", current_priority);
#endif
      exit (0);
    }

  errno = 0;
#ifndef NICE_PRIORITY
  current_priority = getpriority (PRIO_PROCESS, 0);
#else
  current_priority = nice (0);
#endif
  if (current_priority == -1 && errno != 0)
    error (1, errno, "cannot get priority");

#ifndef NICE_PRIORITY
  if (setpriority (PRIO_PROCESS, 0, current_priority + adjustment))
#else
  if (nice (adjustment) == -1)
#endif
    error (1, errno, "cannot set priority");

  execvp (argv[optind], &argv[optind]);
  error (errno == ENOENT ? 127 : 126, errno, "%s", argv[optind]);
}

/* Return nonzero if S represents a (possibly signed) decimal integer,
   zero if not. */

int
isinteger (s)
     char *s;
{
  if (*s == '-')
    ++s;
  if (*s == 0)
    return 0;
  while (*s)
    {
      if (*s < '0' || *s > '9')
	return 0;
      ++s;
    }
  return 1;
}

void
usage ()
{
  print_version("nice");
#ifdef OS2
  fprintf (stderr, "\
Usage: %s [-irtf] [-n adjustment] [-adjustment] [--adjustment=adjustment]\n\
       [command [arg...]]\n",
	   program_name);
  fprintf (stderr, "\n\
The options -irtf specify the following priority classes:\n\
\n\
-i   idletime\n\
-r   regular\n\
-t   timecritical\n\
-f   foregroundserver\n\
\n\
Note that this program follows the Unix style which is that\n\
negative adjustments increase the priority while positive\n\
adjustments decrease the priority. Normally, all OS/2 processes\n\
run at the lowest priority in their class already, so only\n\
negative values make sense unless priority has been increased\n\
previously, to allow a decrease later.\n");
#else
  fprintf (stderr, "\
Usage: %s [-n adjustment] [-adjustment] [--adjustment=adjustment]\n\
       [command [arg...]]\n",
	   program_name);
#endif
  exit (1);
}
